#define MIME_CHARSET_UNKNOWN			-1
#define MIME_CHARSET_NONE			0
#define MIME_CHARSET_USASCII			1
#define MIME_CHARSET_8859_1			2

#define MIME_TYPE_UNKNOWN			-1
#define MIME_TYPE_NONE				0
#define MIME_TYPE_TEXT_PLAIN			1
#define MIME_TYPE_TEXT_ENRICHED			2
#define MIME_TYPE_TEXT_HTML			3
#define MIME_TYPE_MULTIPART_MIXED		4
#define MIME_TYPE_MULTIPART_SIGNED		5
#define MIME_TYPE_APPLICATION_OCTETSTREAM	6
#define MIME_TYPE_APPLICATION_PGP		7
#define MIME_TYPE_IMAGE_GIF			8
#define MIME_TYPE_IMAGE_XBITMAP			9
 
#define MIME_ENCODE_UNKNOWN			-1
#define MIME_ENCODE_NONE			0
#define MIME_ENCODE_7BIT			1
#define MIME_ENCODE_8BIT			2
#define MIME_ENCODE_QUOTED			3
#define MIME_ENCODE_BASE64			4
#define MIME_ENCODE_UUENCODE			5
