#define HTMLCMD_DOC_START	"<HTML>"
#define HTMLCMD_DOC_END		"</HTML>"
#define HTMLCMD_HEAD_START	"<HEAD>"
#define HTMLCMD_HEAD_END	"</HEAD>"
#define HTMLCMD_BODY_START	"<BODY>"
#define HTMLCMD_BODY_END	"</BODY>"
#define HTMLCMD_TITLE_START	"<TITLE>"
#define HTMLCMD_TITLE_END	"</TITLE>"
#define HTMLCMD_TITLE1_START	"<H1>"
#define HTMLCMD_TITLE1_END	"</H1>"
#define HTMLCMD_TITLE2_START	"<H2>"
#define HTMLCMD_TITLE2_END	"</H2>"
#define HTMLCMD_TITLE3_START	"<H3>"
#define HTMLCMD_TITLE3_END	"</H3>"
#define HTMLCMD_TITLE4_START	"<H4>"
#define HTMLCMD_TITLE4_END	"</H4>"
#define HTMLCMD_TITLE5_START	"<H5>"
#define HTMLCMD_TITLE5_END	"</H5>"
#define HTMLCMD_TITLE6_START	"<H6>"
#define HTMLCMD_TITLE6_END	"</H6>"
#define HTMLCMD_REMARK_START	"<!--"
#define HTMLCMD_REMARK_END	"-->"
#define HTMLCMD_PRE_START	"<PRE>"
#define HTMLCMD_PRE_END		"</PRE>"
#define HTMLCMD_BOLD_START	"<B>"
#define HTMLCMD_BOLD_END	"</B>"
#define HTMLCMD_ITALIC_START	"<I>"
#define HTMLCMD_ITALIC_END	"</I>"
#define HTMLCMD_HR_START	"<HR>"
#define HTMLCMD_HR_END		"</HR>"
